# Copyright 2023 (c) Anna Schumaker.
"""Our Testcase Gio.ListModel."""
import typing
from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Gtk
from .. import sqlite


class XunitResult(GObject.GObject):
    """The results for a single TestCase with a specific Xunit."""

    name = GObject.Property(type=str)
    status = GObject.Property(type=str)
    time = GObject.Property(type=int)
    message = GObject.Property(type=str)
    stdout = GObject.Property(type=str)
    stderr = GObject.Property(type=str)


class TestCase(GObject.GObject):
    """Collects results for a single TestCase with multiple Xunits."""

    name = GObject.Property(type=str)

    def __init__(self, name: str) -> None:
        """Initialize a TestCase object."""
        super().__init__(name=name)
        self.__xunits = {}

    def __getitem__(self, xunit: str) -> XunitResult | None:
        """Get the results for a specific Xunit."""
        return self.__xunits.get(xunit)

    def __lt__(self, rhs: typing.Self) -> bool:
        """Compare the names of two TestCases."""
        return self.name < rhs.name

    def add_xunit(self, name: str, status: str, time: int,
                  message: str | None, stdout: str | None,
                  stderr: str | None) -> None:
        """Add an xunit result to the TestCase."""
        message = "" if message is None else message
        stdout = "" if stdout is None else stdout
        stderr = "" if stderr is None else stderr
        self.__xunits[name] = XunitResult(name=name, status=status, time=time,
                                          message=message, stdout=stdout,
                                          stderr=stderr)

    def get_results(self) -> set[str]:
        """Get a set of results for each added xunit."""
        return {xunit.status for xunit in self.__xunits.values()}


class TestCaseList(GObject.GObject, Gio.ListModel):
    """A list of TestCases for a specific Xfstests Run."""

    runid = GObject.Property(type=int)
    n_items = GObject.Property(type=int)

    def __init__(self, sql: sqlite.Connection, runid: int) -> None:
        """Initialize a TestCaseList."""
        super().__init__(runid=runid)
        self.__xunits = set()

        cases = {}
        cur = sql("""SELECT testcase, xunit, status,
                            time, message, stdout, stderr
                  FROM testcases_view WHERE runid=?""", runid)
        for row in cur.fetchall():
            testcase = cases.setdefault(row["testcase"],
                                        TestCase(row["testcase"]))
            testcase.add_xunit(row["xunit"], row["status"], row["time"],
                               row["message"], row["stdout"], row["stderr"])
            self.__xunits.add(row["xunit"])

        self.__items = sorted(cases.values())
        self.n_items = len(self.__items)

    def do_get_item_type(self) -> GObject.GType:
        """Get the type of the objects in the list."""
        return TestCase.__gtype__

    def do_get_n_items(self) -> int:
        """Get the number of items in the list."""
        return self.n_items

    def do_get_item(self, n: int) -> TestCase | None:
        """Get a specific item on the list."""
        return self.__items[n] if n < self.n_items else None

    def get_xunits(self) -> list[str]:
        """Get a list of xunits  attached to this xfstests run."""
        return sorted(self.__xunits)


class TestCaseFilter(Gtk.Filter):
    """A filter for TestCases."""

    passed = GObject.Property(type=bool, default=False)
    skipped = GObject.Property(type=bool, default=False)
    failure = GObject.Property(type=bool, default=True)

    def __init__(self) -> None:
        """Initialize a TestCaseFilter."""
        super().__init__()
        self.connect("notify", self.__notify)

    def __notify(self, filter: typing.Self, param: GObject.ParamSpec) -> None:
        match param.name:
            case "passed" | "skipped" | "failure":
                if self.get_property(param.name):
                    change = Gtk.FilterChange.LESS_STRICT
                else:
                    change = Gtk.FilterChange.MORE_STRICT
                self.changed(change)

    def do_get_strictness(self) -> Gtk.FilterMatch:
        """Get the current strictness of the filter."""
        match (self.passed, self.skipped, self.failure):
            case (True, True, True): return Gtk.FilterMatch.ALL
            case (False, False, False): return Gtk.FilterMatch.NONE
            case (_, _, _): return Gtk.FilterMatch.SOME

    def do_match(self, testcase: TestCase) -> bool:
        """Check if a testcase matches the current filter."""
        results = testcase.get_results()
        if self.passed and "passed" in results:
            return True
        if self.skipped and "skipped" in results:
            return True
        if self.failure and "failure" in results:
            return True
        return False


class SummaryValue(GObject.GObject):
    """The summary of a single Xfstests xunit field."""

    name = GObject.Property(type=str)
    unit = GObject.Property(type=str)
    value = GObject.Property(type=int)

    def __str__(self) -> str:
        """Convert a Summary Value to a string."""
        s = '' if self.value == 1 else 's'
        return f"{self.value} {self.unit}{s}"


class Summary(GObject.GObject):
    """Collects values for each summary field with multiple Xunits."""

    name = GObject.Property(type=str)

    def __init__(self, name: str) -> None:
        """Initialize a Summary object."""
        super().__init__(name=name)
        self.__xunits = {}

    def __getitem__(self, xunit: str) -> SummaryValue | None:
        """Get the summary for a specific Xunit."""
        return self.__xunits.get(xunit)

    def __lt__(self, rhs: typing.Self) -> bool:
        """Compare the fields of two Summaries."""
        order = ["passed", "failed", "skipped", "time"]
        return order.index(self.name) < order.index(rhs.name)

    def add_xunit(self, name: str, value: int, unit: str) -> None:
        """Add an xunit summary to the Summary."""
        self.__xunits[name] = SummaryValue(name=name, value=value, unit=unit)

    def get_results(self) -> set[str]:
        """Get a set of results for each added xunit."""
        return {str(value) for value in self.__xunits.values()}


class SummaryList(GObject.GObject, Gio.ListModel):
    """A list summarizing the results of a specific Xfstests Run."""

    runid = GObject.Property(type=int)
    n_items = GObject.Property(type=int)

    def __init__(self, sql: sqlite.Connection, runid: int) -> None:
        """Initialize a SummaryList."""
        super().__init__(runid=runid)
        self.__xunits = set()

        results = {}
        cur = sql("""SELECT name AS xunit, passed, failed, skipped, time
                  FROM xunits_view WHERE runid=?""", runid)
        for row in cur.fetchall():
            for field in ["passed", "failed", "skipped", "time"]:
                summary = results.setdefault(field, Summary(field))
                summary.add_xunit(row["xunit"], row[field],
                                  "second" if field == "time" else "testcase")
            self.__xunits.add(row["xunit"])

        self.__items = sorted(results.values())
        self.n_items = len(self.__items)

    def do_get_item_type(self) -> GObject.GType:
        """Get the type of the objects in the list."""
        return Summary.__gtype__

    def do_get_n_items(self) -> int:
        """Get the number of items in the list."""
        return self.n_items

    def do_get_item(self, n: int) -> Summary | None:
        """Get a specific item on the list."""
        return self.__items[n] if n < self.n_items else None

    def get_xunits(self) -> list[str]:
        """Get a list of xunits  attached to this xfstests run."""
        return sorted(self.__xunits)
