# Copyright 2023 (c) Anna Schumaker.
"""The `xfstestsdb xunit list` command."""
import argparse
from .. import colors
from .. import commands
from .. import sqlite
from .. import table


class Command(commands.Command):
    """The `xfstestsdb xunit list` command."""

    def __init__(self, subparser: argparse.Action,
                 sql: sqlite.Connection) -> None:
        """Set up the list command."""
        super().__init__(subparser, sql, "list",
                         help="list the xfstest xunit entries")
        self.parser.add_argument("--color", metavar="color", nargs="?",
                                 choices=["light", "dark", "none"],
                                 const=colors.get_default_colors(),
                                 default=colors.get_default_colors(),
                                 help="show xunits with color output "
                                 f"[default={colors.get_default_colors()}]")
        self.parser.add_argument("--device", metavar="device", nargs=1,
                                 help="show xunits with a device matching "
                                 "the given pattern")
        self.parser.add_argument("--hostname", metavar="hostname", nargs=1,
                                 help="show xunits with a hostname matching "
                                 "the given pattern")
        self.parser.add_argument("--results", action="store_true",
                                 help="show the results columns")
        self.parser.add_argument("--runid", metavar="runid", nargs=1, type=int,
                                 help="show xunits belonging to the given run")
        self.parser.add_argument("--timestamp", metavar="timestamp", nargs="?",
                                 const=True, default=False,
                                 help="show xunits with a timestamp matching "
                                 "the given pattern")
        self.parser.add_argument("--xunit", metavar="xunit", nargs=1,
                                 help="show xunits with a name "
                                 "matching the given pattern")

    def do_command(self, args: argparse.Namespace) -> None:
        """Print out the xfstestsdb xunits table."""
        tbl = table.Table(["run", "device", "xunit", "hostname"],
                          ["r", "l", "l", "l"], args.color)
        sql_where = ""
        sql_args = []
        filter = []

        if args.device:
            filter.append("device GLOB ?")
            sql_args.append(args.device[0])

        if args.hostname:
            filter.append("hostname GLOB ?")
            sql_args.append(args.hostname[0])

        if args.timestamp:
            tbl.add_column("timestamp", "c")
            if isinstance(args.timestamp, str):
                filter.append("timestamp GLOB ?")
                sql_args.append(args.timestamp)

        if args.results:
            tbl.add_column("pass", "r")
            tbl.add_column("fail", "r")
            tbl.add_column("skip", "r")
            tbl.add_column("time", "r")

        if args.runid:
            filter.append("runid GLOB ?")
            sql_args.append(args.runid[0])

        if args.xunit:
            filter.append("name GLOB ?")
            sql_args.append(args.xunit[0])

        if len(filter) > 0:
            sql_where = " WHERE " + " AND ".join(filter) + " "

        cur = self.sql("""SELECT runid, device, name, hostname, timestamp,
                                 tests, passed, failed, skipped, time
                       FROM xunits_view""" + sql_where + """
                       ORDER BY runid, name""", *sql_args)
        for row in cur.fetchall():
            data = [row["runid"], row["device"], row["name"], row["hostname"]]
            if args.timestamp:
                data.append(row["timestamp"])
            if args.results:
                data.extend([row["passed"], row["failed"], row["skipped"],
                            f"{row['time']} s"])
            tbl.add_row(*data)

        if len(tbl.rows) > 0:
            print(tbl)
